require "import"
import "android.widget.*"
import "android.content.*"
import "android.os.Vibrator"
import "android.os.VibrationEffect"
import "android.os.Build"
import "android.view.*"
import "android.graphics.Color"
import "android.graphics.drawable.ColorDrawable"
import "com.androlua.LuaDialog"
import "java.lang.Runnable"

-- ========== CONFIGURACIÓN INICIAL ==========
local context = service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
-- Variable para saber si la vibración debería seguir activa
local vibracionActiva = false

-- ========== PATRONES DE VIBRACIÓN ==========
local vibrationPatterns = {
  baja              = {100, 200},
  media             = {200, 150, 200},
  placentera        = {300, 100, 300},
  rotatorio         = {100, 50, 100, 50, 100, 50, 100},
  lineal            = {400, 400, 400},
  pulsante          = {150, 100, 150, 100, 150, 100, 150},
  vibradorSuave     = {80, 300, 80, 300, 80, 300},
  vibradorPulsante  = {200, 100, 200, 100, 200, 100},
  vibradorIntenso   = {500, 50, 500, 50, 500, 50},
  vibradorOndulante = {300, 150, 400, 150, 300, 150, 200, 150},
  consoladorRitmico = {150, 150, 150, 150, 150, 150},
  consoladorMasajista = {100, 100, 200, 100, 300, 100, 200, 100}
}

-- Clase correcta para long[]
local LongClass = luajava.bindClass("java.lang.Long")

-- ========== FUNCIÓN DE VIBRACIÓN ==========
local function vibrar(nombre, patron)
  if vibrator and vibrator.hasVibrator() then
    vibracionActiva = true -- Marcar que la vibración está activa
    local javaArray = luajava.newArray(LongClass.TYPE, #patron)
    for i = 1, #patron do
      javaArray[i - 1] = tonumber(patron[i])
    end
    if Build.VERSION.SDK_INT >= 26 then
      local effect = VibrationEffect.createWaveform(javaArray, 0) -- Repetir infinitamente
      vibrator.vibrate(effect)
    else
      vibrator.vibrate(javaArray, 0)
    end
    service.speak("Modo: " .. nombre)
  else
    service.speak("Vibrador no disponible")
  end
end

-- ========== FUNCIÓN PARA DETENER LA VIBRACIÓN ==========
local function detenerVibracion()
  if vibrator then
    vibrator.cancel()
    vibracionActiva = false -- Marcar que la vibración se ha detenido
    service.speak("Vibración detenida")
  end
end

-- ========== FUNCIÓN PARA MONITOREAR LA VIBRACIÓN ==========
local function verificarVibracion()
  task(1000, function() -- Revisar cada 1 segundo
    if vibracionActiva and not vibrator.hasVibrator() then
      service.speak("Reiniciando vibración...")
      for nombre, patron in pairs(vibrationPatterns) do
        vibrar(nombre, patron)
        break -- Reiniciar con el primer patrón encontrado
      end
    end
    if vibracionActiva then
      verificarVibracion() -- Seguir revisando si la vibración sigue activa
    end
  end)
end

-- Iniciar monitoreo de vibración
verificarVibracion()

-- ========== INTERFAZ ==========
function crearInterfaz()
  local dlg = LuaDialog(service)
  local window = dlg.getWindow()
  if window then
    window.setBackgroundDrawable(ColorDrawable(Color.parseColor("#212121")))
  end
  local layout = LinearLayout(service)
  layout.setOrientation(LinearLayout.VERTICAL)
  layout.setPadding(24, 24, 24, 24)

  local titulo = TextView(service)
  titulo.setText("Vibrador Virtual")
  titulo.setTextSize(24)
  titulo.setTextColor(Color.parseColor("#00BCD4"))
  titulo.setGravity(Gravity.CENTER)

  local subtitulo = TextView(service)
  subtitulo.setText("Simulaciones de vibradores y consoladores")
  subtitulo.setTextSize(14)
  subtitulo.setTextColor(Color.parseColor("#757575"))
  subtitulo.setGravity(Gravity.CENTER)
  subtitulo.setPadding(0, 0, 0, 32)

  layout.addView(titulo)
  layout.addView(subtitulo)

  local scroll = ScrollView(service)
  local contenedor = LinearLayout(service)
  contenedor.setOrientation(LinearLayout.VERTICAL)
  scroll.addView(contenedor)

  -- Lista de botones con diversidad de patrones
  local botones = {
    {nombre = "Baja",              color = "#4CAF50",  patron = "baja"},
    {nombre = "Media",             color = "#FFC107",  patron = "media"},
    {nombre = "Placentera",        color = "#9C27B0",  patron = "placentera"},
    {nombre = "Rotatorio",         color = "#00BCD4",  patron = "rotatorio"},
    {nombre = "Lineal",            color = "#2196F3",  patron = "lineal"},
    {nombre = "Pulsante",          color = "#F44336",  patron = "pulsante"},
    {nombre = "Vibrador Suave",    color = "#8BC34A",  patron = "vibradorSuave"},
    {nombre = "Vibrador Pulsante", color = "#FF9800",  patron = "vibradorPulsante"},
    {nombre = "Vibrador Intenso",  color = "#E91E63",  patron = "vibradorIntenso"},
    {nombre = "Vibrador Ondulante",color = "#3F51B5",  patron = "vibradorOndulante"},
    {nombre = "Consolador Rítmico",color = "#9E9E9E",  patron = "consoladorRitmico"},
    {nombre = "Consolador Masajista", color = "#795548", patron = "consoladorMasajista"}
  }

  for _, btn in ipairs(botones) do
    local boton = Button(service)
    boton.setText(btn.nombre)
    boton.setTextSize(18)
    boton.setTextColor(Color.WHITE)
    boton.setBackground(ColorDrawable(Color.parseColor(btn.color)))
    boton.setPadding(40, 24, 40, 24)
    local params = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
    params.bottomMargin = 16
    boton.setLayoutParams(params)
    boton.onClick = function()
      vibrar(btn.nombre, vibrationPatterns[btn.patron])
    end
    contenedor.addView(boton)
  end

  layout.addView(scroll)

  local botonDetener = Button(service)
  botonDetener.setText("Detener Vibración")
  botonDetener.setTextSize(18)
  botonDetener.setTextColor(Color.WHITE)
  botonDetener.setBackground(ColorDrawable(Color.parseColor("#E53935")))
  botonDetener.setPadding(40, 24, 40, 24)
  local paramsDetener = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
  paramsDetener.topMargin = 16
  paramsDetener.bottomMargin = 16
  botonDetener.setLayoutParams(paramsDetener)
  botonDetener.onClick = function()
    detenerVibracion()
  end
  layout.addView(botonDetener)

  local botonSalir = Button(service)
  botonSalir.setText("Salir")
  botonSalir.setTextSize(18)
  botonSalir.setTextColor(Color.WHITE)
  botonSalir.setBackground(ColorDrawable(Color.parseColor("#616161")))
  botonSalir.setPadding(40, 24, 40, 24)
  local paramsSalir = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
  botonSalir.setLayoutParams(paramsSalir)
  botonSalir.onClick = function()
    dlg.dismiss()
  end
  layout.addView(botonSalir)

  dlg.setView(layout)
  dlg.show()
end

-- ========== INICIALIZACIÓN ==========
if vibrator and vibrator.hasVibrator() then
  crearInterfaz()
else
  service.speak("Error: Vibrador no disponible")
  service.stop()
end
